@@ -1,3 +1,4 @@
+{-# OPTIONS -fffi #-}
 -----------------------------------------------------------------------------
 -- |
 -- Module      :  FastPackedString
@@ -20,8 +21,10 @@
 module FastPackedString (
 	-- * The @PackedString@ type
         PackedString,      -- abstract, instances: Eq, Ord, Show, Typeable
+        unsafeWithInternals, -- :: PackedString -> (Ptr Word8 -> Int -> IO a) -> IO a
 
          -- * Converting to and from @PackedString@s
+    generatePS,  -- :: Int -> (Ptr Word8 -> Int -> IO Int) -> IO PackedString
 	packString,  -- :: String -> PackedString
 	packWords,   -- :: [Word8] -> PackedString
 	unpackPS,    -- :: PackedString -> String
@@ -31,6 +34,7 @@ module FastPackedString (
 	-- * I\/O with @PackedString@s
 	hPutPS,      -- :: Handle -> PackedString -> IO ()
 	hGetPS,      -- :: Handle -> Int -> IO PackedString
+    hGetContentsPS, -- :: Handle -> IO PackedString
 	readFilePS,  -- :: FilePath -> IO PackedString
 	writeFilePS, -- :: FilePath -> PackedString -> IO ()
 	gzReadFilePS,-- :: FilePath -> IO PackedString
@@ -41,6 +45,7 @@ module FastPackedString (
 	consPS,      -- :: Char -> PackedString -> PackedString
 	headPS,      -- :: PackedString -> Char
 	tailPS,      -- :: PackedString -> PackedString
+	lastPS,      -- :: PackedString -> Char
 	nullPS,      -- :: PackedString -> Bool
 	appendPS,    -- :: PackedString -> PackedString -> PackedString
 	lengthPS,    -- :: PackedString -> Int
@@ -50,6 +55,7 @@ module FastPackedString (
 	--filterPS,    -- :: (Char -> Bool) -> PackedString -> PackedString
 	reversePS,   -- :: PackedString -> PackedString
 	concatPS,    -- :: [PackedString] -> PackedString
+	concatLenPS, -- :: Int -> [PackedString] -> PackedString
 	elemPS,      -- :: Char -> PackedString -> Bool
 	takePS,      -- :: Int -> PackedString -> PackedString
 	dropPS,      -- :: Int -> PackedString -> PackedString
@@ -82,6 +88,7 @@ module FastPackedString (
         is_funky,    -- :: PackedString -> Bool
         fromHex2PS,  -- :: PackedString -> PackedString
         fromPS2Hex,  -- :: PackedString -> PackedString
+        betweenLinesPS,--  :: PackedString -> PackedString -> PackedString -> Maybe (PackedString)
     ) where
 
 import IO ( Handle, hClose, hFileSize, IOMode(ReadMode,WriteMode),
@@ -93,12 +100,11 @@ import System.Mem ( performGC )
 import Foreign.Storable ( peekElemOff, peek, poke )
 import Ptr ( nullPtr, plusPtr, minusPtr, Ptr )
 import Foreign.Marshal.Alloc ( free )
-import Foreign.Marshal.Array ( pokeArray, mallocArray, peekArray )
+import Foreign.Marshal.Array ( pokeArray, mallocArray, reallocArray,
+                               peekArray, advancePtr )
 import Foreign.Marshal.Utils ( with )
-import Foreign.ForeignPtr ( addForeignPtrFinalizer )
 import Foreign.C.String
-import Foreign.C.Types ( CLong )
-import Data.Dynamic
+import Foreign.C.Types ( CLong, CInt, CSize )
 import Data.Char
 import Data.Word
 import Monad ( liftM, when )
@@ -107,35 +113,18 @@ import Workaround ( handleToFd, fdToInt )
 import System.IO.Unsafe ( unsafePerformIO )
 import System.IO ( hPutBuf, hGetBuf )
 
-#if __GLASGOW_HASKELL__ > 601
 import System.IO ( openBinaryFile )
-#else
-import GHC.Handle ( openFileEx, IOModeEx(..) )
-#endif
 
-#if __GLASGOW_HASKELL__ < 600
-import Foreign.ForeignPtr( ForeignPtr, withForeignPtr, newForeignPtr )
-mallocForeignPtrArray :: Int -> IO (ForeignPtr Word8)
-mallocForeignPtrArray i = do p <- mallocArray i
-                             mallocedPtrToForeignPtr p
-#else
 import Foreign.ForeignPtr( ForeignPtr, withForeignPtr, mallocForeignPtrArray )
 import Foreign.Concurrent( newForeignPtr )
-import qualified Foreign.ForeignPtr( newForeignPtr )
+
+#ifdef DEBUG_PS
+import Foreign.ForeignPtr ( addForeignPtrFinalizer )
 import Foreign.Ptr ( FunPtr )
-import Foreign.Marshal.Alloc( finalizerFree )
 #endif
 
 debugForeignPtr :: ForeignPtr a -> String -> IO ()
-mallocedPtrToForeignPtr :: Ptr a -> IO (ForeignPtr a)
--- The argument to mallocedPtrToForeignPtr *must* have been allocated with
--- malloc or friends.
-#if __GLASGOW_HASKELL__ < 600
-debugForeignPtr fp n =
-    do putStr $ "Allocating "++n++"\n"
-       addForeignPtrFinalizer fp (putStr $ "Freeing "++n++"\n")
-mallocedPtrToForeignPtr p = newForeignPtr p (free p)
-#else
+#ifdef DEBUG_PS
 foreign import ccall unsafe "static fpstring.h debug_alloc" debug_alloc
     :: Ptr a -> CString -> IO ()
 foreign import ccall unsafe "static fpstring.h & debug_free" debug_free
@@ -143,19 +132,9 @@ foreign import ccall unsafe "static fpstring.h & debug_free" debug_free
 debugForeignPtr fp n =
     withCString n $ \cname-> withForeignPtr fp $ \p->
     do debug_alloc p cname
-#if __GLASGOW_HASKELL__ > 601
        addForeignPtrFinalizer debug_free fp
-mallocedPtrToForeignPtr p =
-    Foreign.ForeignPtr.newForeignPtr finalizerFree p
 #else
-       addForeignPtrFinalizer fp debug_free
-mallocedPtrToForeignPtr p =
-    Foreign.ForeignPtr.newForeignPtr p finalizerFree
-#endif
-#endif
-
-#if __GLASGOW_HASKELL__ < 601
-openBinaryFile f m = openFileEx f (BinaryMode m)
+debugForeignPtr _ _ = return ()
 #endif
 
 foreign import ccall unsafe "static fpstring.h dumb_malloc" dumb_malloc
@@ -177,9 +156,11 @@ mallocForeignPtr l
 --isn't entirely "safe", but at least it's convenient.
 
 createPS :: Int -> (Ptr Word8 -> IO ()) -> PackedString
-createPS l write_ptr = unsafePerformIO $ do fp <- mallocForeignPtr l
-                                            withForeignPtr fp $ \p -> write_ptr p
-                                            return $ PS fp 0 l
+createPS l write_ptr =
+    unsafePerformIO $ do fp <- mallocForeignPtr l
+                         debugForeignPtr fp "createPS"
+                         withForeignPtr fp $ \p -> write_ptr p
+                         return $ PS fp 0 l
 
 -- -----------------------------------------------------------------------------
 -- PackedString type declaration
@@ -188,6 +169,15 @@ createPS l write_ptr = unsafePerformIO $ do fp <- mallocForeignPtr l
 -- efficient operations.  A 'PackedString' contains full Unicode 'Char's.
 data PackedString = PS !(ForeignPtr Word8) !Int !Int
 
+-- -----------------------------------------------------------------------------
+-- unsafeWithInternals
+
+-- | Do something with the internals of a PackedString. Beware of
+-- altering the contents!
+unsafeWithInternals :: PackedString -> (Ptr Word8 -> Int -> IO a) -> IO a
+unsafeWithInternals (PS fp s l) f
+ = withForeignPtr fp $ \p -> f (p `plusPtr` s) l
+
 {-# INLINE (!) #-}
 (!) :: PackedString -> Int -> Word8
 (PS x s _l) ! i
@@ -196,6 +186,21 @@ data PackedString = PS !(ForeignPtr Word8) !Int !Int
   -- | i >= l = error "Out of range element in PackedString."
   -- | otherwise = unsafePerformIO $ withForeignPtr x $ \p -> peekElemOff p (s+i)
 
+-- -----------------------------------------------------------------------------
+-- generatePS
+
+-- | Given the maximum size needed and a function to make the contents
+-- of a PackedString, generatePS makes the PackedString. The generating
+-- function is required to return the actual size (<= the maximum size).
+
+generatePS :: Int -> (Ptr Word8 -> IO Int) -> IO PackedString
+generatePS i f
+ = do p <- mallocArray i
+      i' <- f p
+      p' <- reallocArray p i'
+      fp <- newForeignPtr p' (c_free p')
+      return $ PS fp 0 i'
+
 instance Eq PackedString where
    (==) = psniceq
 
@@ -241,14 +246,12 @@ pscmp (PS x1 s1 l1) (PS x2 s2 l2) =
 instance Show PackedString where
     showsPrec p ps r = showsPrec p (unpackPS ps) r
 
-packedStringTc = mkTyCon "PackedString"
-instance Typeable PackedString where { typeOf _ = mkAppTy packedStringTc [] }
-
 -- -----------------------------------------------------------------------------
 -- Constructor functions
 
 nilPS :: PackedString
 nilPS = unsafePerformIO $ do fp <- mallocForeignPtr 1
+                             debugForeignPtr fp "nilPS"
                              return $ PS fp 0 0
 
 consPS :: Char -> PackedString -> PackedString
@@ -272,13 +275,31 @@ c2w = fromIntegral . ord
 
 -- | Convert a 'PackedString' into a 'String'
 unpackPS :: PackedString -> String
+unpackPS (PS ps s l)
+ = map w2c $ unsafePerformIO
+           $ withForeignPtr ps $ \p -> peekArray l (p `plusPtr` s)
+{-
+unpackPS :: PackedString -> String
+unpackPS theps@(PS ps s l)
+ | l >= 1024 = map w2c (unsafePerformIO (withForeignPtr ps $
+                                        \p -> peekArray 1024 (p `plusPtr` s)))
+            ++ unpackPS (PS ps (s + 1024) (l - 1024))
+ | l >= 128 = map w2c (unsafePerformIO (withForeignPtr ps $
+                                       \p -> peekArray 128 (p `plusPtr` s)))
+           ++ unpackPS (PS ps (s + 128) (l - 128))
+ | l > 0 = unsafeHeadPS theps : unpackPS (unsafeTailPS theps)
+ | otherwise = ""
+-}
+{-
+unpackPS :: PackedString -> String
 unpackPS theps = if nullPS theps then []
-                 else headPS theps : unpackPS (tailPS theps)
+                 else unsafeHeadPS theps : unpackPS (unsafeTailPS theps)
+-}
 unpackWords :: PackedString -> [Word8]
 unpackWords ps@(PS x s _) =
     if nullPS ps then []
     else (unsafePerformIO $ withForeignPtr x $ \p -> peekElemOff p s)
-             : unpackWords (tailPS ps)
+             : unpackWords (unsafeTailPS ps)
 
 unpackPSfromUTF8 :: PackedString -> String
 unpackPSfromUTF8 (PS _ _ 0) = ""
@@ -313,12 +334,24 @@ indexPS theps i | i < 0 = error "Negative index in indexPS"
                 | i >= lengthPS theps = error "Out of bounds in indexPS"
                 | otherwise = w2c $ theps ! i
 
+{-# INLINE lastPS #-}
+lastPS :: PackedString -> Char
+lastPS ps@(PS x s l) -- ps ! 0 is inlined manually to eliminate a (+0)
+  | nullPS ps = error "FastPackedString.lastPS: last []"
+  | otherwise  = w2c $ unsafePerformIO $ withForeignPtr x $
+                 \p -> peekElemOff p (s+l-1)
+
 {-# INLINE headPS #-}
 headPS :: PackedString -> Char
 headPS ps@(PS x s _) -- ps ! 0 is inlined manually to eliminate a (+0)
   | nullPS ps = error "FastPackedString.headPS: head []"
   | otherwise  = w2c $ unsafePerformIO $ withForeignPtr x $ \p -> peekElemOff p s
 
+{-# INLINE unsafeHeadPS #-}
+unsafeHeadPS :: PackedString -> Char
+unsafeHeadPS (PS x s _) -- ps ! 0 is inlined manually to eliminate a (+0)
+  = w2c $ unsafePerformIO $ withForeignPtr x $ \p -> peekElemOff p s
+
 {-# INLINE tailPS #-}
 tailPS :: PackedString -> PackedString
 tailPS ps
@@ -328,6 +361,12 @@ tailPS ps
   where
     len = lengthPS ps
 
+{-# INLINE unsafeTailPS #-}
+unsafeTailPS :: PackedString -> PackedString
+unsafeTailPS (PS ps s l)
+  | l == 1 = nilPS
+  | otherwise  = PS ps (s+1) (l-1)
+
 {-# INLINE nullPS #-}
 nullPS :: PackedString -> Bool
 nullPS (PS _ _ l) = l == 0
@@ -339,7 +378,8 @@ appendPS xs ys
   | otherwise  = concatPS [xs,ys]
 
 mapPS :: (Char -> Char) -> PackedString -> PackedString
-mapPS func (PS ps _ l) = createPS l $ \p-> withForeignPtr ps $ \f-> mint f p l
+mapPS func (PS ps s l) = createPS l $ \p-> withForeignPtr ps $
+                         \f-> mint (f `plusPtr` s) p l
     where mint :: Ptr Word8 -> Ptr Word8 -> Int -> IO ()
           mint _ _ 0 = return ()
           mint f t len = do val <- peek f
@@ -357,7 +397,7 @@ foldrPS f v ps = foldr f v (unpackPS ps)
 
 {-# INLINE takePS #-}
 takePS :: Int -> PackedString -> PackedString
-takePS n ps@(PS x s _) = if n > lengthPS ps then ps
+takePS n ps@(PS x s _) = if n >= lengthPS ps then ps
                          else PS x s n -- substrPS ps 0 (n - 1)
 
 {-# INLINE dropPS #-}
@@ -372,19 +412,21 @@ splitAtPS  n ps  = (takePS n ps, dropPS n ps)
 
 -- This must be fast, it's used heavily in Printer. -- jch
 anyPS :: (Char -> Bool) -> PackedString -> Bool
-anyPS f (PS x s' l) = a s'
-    where
-    a s | s >= l = False
-    a s | f $ aref s = True
-    a s = a (s + 1)
-    aref n = w2c $ unsafePerformIO $ withForeignPtr x $ 
-             \p -> peekElemOff p n
+anyPS f (PS x s l) =
+    unsafePerformIO $ withForeignPtr x $ \ptr ->
+        lookat (ptr `plusPtr` s) (ptr `plusPtr` (s+l))
+    where lookat :: Ptr Word8 -> Ptr Word8 -> IO Bool
+          lookat p st | p == st = return False
+                      | otherwise = do w <- peek p
+                                       if f $ w2c w
+                                          then return True
+                                          else lookat (p `plusPtr` 1) st
 
 findWhenPS :: (Char -> Bool) -> PackedString -> Int
 findWhenPS f ps = seq f $
     if nullPS ps then 0
-    else if f $ headPS ps then 0
-         else 1 + findWhenPS f (tailPS ps)
+    else if f $ unsafeHeadPS ps then 0
+         else 1 + findWhenPS f (unsafeTailPS ps)
 
 {-# INLINE takeWhilePS #-}
 takeWhilePS :: (Char -> Bool) -> PackedString -> PackedString
@@ -478,9 +520,34 @@ foreign import ccall unsafe "static string.h memcpy" c_memcpy
 concatPS :: [PackedString] -> PackedString
 concatPS [] = nilPS
 concatPS [ps] = ps
-concatPS pss = createPS total_length $ \p-> cpPSs p pss
-    where total_length = sum $ map lengthPS pss
-          cpPSs :: Ptr Word8 -> [PackedString] -> IO ()
+concatPS xs
+ = unsafePerformIO $
+   do let start_size = 1024
+      p <- mallocArray start_size
+      f p 0 1024 xs
+    where f ptr len _ [] = do ptr' <- reallocArray ptr len
+                              fp <- newForeignPtr ptr' (c_free ptr')
+                              return $ PS fp 0 len
+          f ptr len to_go pss@(PS p s l:pss')
+           | l <= to_go = do withForeignPtr p $ \pf ->
+                                 c_memcpy (ptr `advancePtr` len)
+                                          (pf `advancePtr` s) l
+                             f ptr (len + l) (to_go - l) pss'
+           | otherwise = do let new_total = ((len + to_go) * 2) `max` (len + l)
+                            ptr' <- reallocArray ptr new_total
+                            f ptr' len (new_total - len) pss
+
+-- -----------------------------------------------------------------------------
+-- concatLenPS
+
+-- | Camse as concatPS only you tell it how big the result will be.
+-- If you lie thenBad Things will happen.
+
+concatLenPS :: Int -> [PackedString] -> PackedString
+concatLenPS n [] = n `seq` nilPS
+concatLenPS _ [ps] = ps
+concatLenPS total_length pss = createPS total_length $ \p-> cpPSs p pss
+    where cpPSs :: Ptr Word8 -> [PackedString] -> IO ()
           cpPSs p (PS x s l:rest) = do withForeignPtr x $ \pf ->
                                           c_memcpy p (pf `plusPtr` s) l
                                        cpPSs (p `plusPtr` l) rest
@@ -494,12 +561,13 @@ findPS c ps = wfindPS (c2w c) ps
 wfindPS :: Word8 -> PackedString -> Maybe Int
 wfindPS c (PS x s l) =
     unsafePerformIO $ withForeignPtr x $ \p->
-    case wfindps_helper c (p `plusPtr` s) l of
-      -1 -> return Nothing
-      i  -> return $ Just i
+    let p' = p `plusPtr` s
+        q = memchr p' (fromIntegral c) (fromIntegral l)
+    in return $ if q == nullPtr then Nothing
+                                else Just (q `minusPtr` p')
 
-foreign import ccall unsafe "fpstring.h wfindps_helper" wfindps_helper
-    :: Word8 -> Ptr Word8 -> Int -> Int
+foreign import ccall unsafe "string.h memchr" memchr
+    :: Ptr Word8 -> CInt -> CSize -> Ptr Word8
 
 {-# INLINE findLastPS #-}
 findLastPS :: Char -> PackedString -> Maybe Int
@@ -573,9 +641,37 @@ hPutPS h (PS ps s l) = withForeignPtr ps $ \p-> hPutBuf h (p `plusPtr` s) l
 hGetPS :: Handle -> Int -> IO PackedString
 hGetPS _ 0 = return nilPS
 hGetPS h i = do fp <- mallocForeignPtr i
+                debugForeignPtr fp $ "hGetPS "++show h
                 l <- withForeignPtr fp $ \p-> hGetBuf h p i
                 return $ PS fp 0 l
 
+-- -----------------------------------------------------------------------------
+-- hGetContentsPS
+
+-- | Read entire handle contents into a 'PackedString'.
+--
+-- NOTE: as with 'hGetPS', the string representation in the file is
+-- assumed to be ISO-8859-1.
+
+hGetContentsPS :: Handle -> IO PackedString
+hGetContentsPS h
+ = do let start_size = 1024
+      p <- mallocArray start_size
+      i <- hGetBuf h p start_size
+      if i < start_size
+       then do p' <- reallocArray p i
+               fp <- newForeignPtr p' (c_free p')
+               return $ PS fp 0 i
+       else f p start_size
+    where f p s = do let s' = 2 * s
+                     p' <- reallocArray p s'
+                     i <- hGetBuf h (p' `plusPtr` s) s
+                     if i < s then do let i' = s + i
+                                      p'' <- reallocArray p' i'
+                                      fp <- newForeignPtr p'' (c_free p'')
+                                      return $ PS fp 0 i'
+                              else f p' s'
+
 -- -----------------------------------------------------------------------------
 -- readFilePS
 
@@ -631,6 +727,7 @@ gzReadFilePS f = do
                  gzf <- c_gzopen fstr rb
                  when (gzf == nullPtr) $ fail $ "problem opening file "++f
                  fp <- mallocForeignPtr len
+                 debugForeignPtr fp $ "gzReadFilePS "++f
                  lread <- withForeignPtr fp $ \p -> c_gzread gzf p len
                  c_gzclose gzf
                  when (lread /= len) $ fail $ "problem gzreading file "++f
@@ -679,6 +776,7 @@ mmap f = do
     -- at least one full VM block.
     if l < mmap_limit
        then do thefp <- mallocForeignPtr l
+               debugForeignPtr thefp $ "mmap short file "++f
                withForeignPtr thefp $ \p-> hGetBuf h p l
                hClose h
                return (thefp, l)
@@ -686,13 +784,13 @@ mmap f = do
                p <- my_mmap l fd
                fp <- if p == nullPtr
                      then do thefp <- mallocForeignPtr l
+                             debugForeignPtr thefp $ "mmap short file "++f
                              withForeignPtr thefp $ \p' -> hGetBuf h p' l
                              return thefp
                      else do fp <- newForeignPtr p (do {c_munmap p l; return (); })
+                             debugForeignPtr fp $ "mmap "++f
                              return fp
-#if __GLASGOW_HASKELL__ >= 600
                c_close fd
-#endif
                hClose h
                return (fp, l)
     where mmap_limit = 16*1024
@@ -753,3 +851,19 @@ mylexPS s = if nullPS (dropWhitePS s)
             then Nothing
             else Just $ breakWhitePS $ dropWhitePS s
 
+-- -------------------------------------------------------------------------
+-- betweenLinesPS
+
+-- | betweenLinesPS returns the PackedString between the two lines given,
+-- or Nothing if they do not appear.
+
+betweenLinesPS :: PackedString -> PackedString -> PackedString
+               -> Maybe (PackedString)
+betweenLinesPS start end ps
+ = case break (start ==) (linesPS ps) of
+       (_, _:rest@(PS ps1 s1 _:_)) ->
+           case break (end ==) rest of
+               (_, PS _ s2 _:_) -> Just $ PS ps1 s1 (s2 - s1)
+               _ -> Nothing
+       _ -> Nothing
+
