@@ -26,20 +26,41 @@ main = man the_commands
 
 man :: [DarcsCommand] -> IO ()
 man cs = do man_header
-            man_helper cs
+            unorganized <- man_organizer cs
+            putStrLn ".SH OTHER COMMANDS"
+            man_helper unorganized
             man_trailer
 
+man_organizer :: [DarcsCommand] -> IO [DarcsCommand]
+man_organizer commands = mo commands
+                         [("CREATING REPOSITORIES",
+                           ["initialize","get"]),
+                          ("MODIFYING REPOSITORY CONTENTS",
+                           ["add","remove","mv","replace"]),
+                          ("WORKING WITH CHANGES",
+                           ["record","pull","push","send","apply"]),
+                          ("SEEING WHAT YOU'VE DONE",
+                           ["whatsnew","changes"])
+                         ]
+    where mo :: [DarcsCommand] -> [(String,[String])] -> IO [DarcsCommand]
+          mo cs [] = return cs
+          mo cs ((t,a):xs) =
+              do putStrLn $ ".SH " ++ t
+                 man_helper $ filter ((`elem` a) . command_name) cs
+                 mo (filter (not . (`elem` a) . command_name) cs) xs
+
 man_helper :: [DarcsCommand] -> IO ()
 man_helper [] = return ()
 man_helper (c:cs) = do
-  putStr $ ".TP\n.B "++command_name c++"\n"
-  putStr $ command_help c++"\n"
+  putStrLn $ ".TP"
+  putStrLn $ ".B "++command_name c
+  putStrLn $ command_help c
   man_helper cs
 
 man_trailer :: IO ()
-man_trailer = putStr $ "\n"++
-                       ".SH AUTHOR\n"++
-                       "David Roundy <droundy@abridgegame.org>.\n"
+man_trailer = do putStrLn ""
+                 putStrLn ".SH AUTHOR"
+                 putStrLn "David Roundy <droundy@abridgegame.org>."
 
 man_header :: IO ()
 man_header = do
@@ -50,7 +71,7 @@ man_header = do
   putStr "\" \"darcs\" \"User Commands\"\n"
   putStr $
    ".SH NAME\n"++
-   "darcs \\- David's Advanced Revision Control System\n"++
+   "darcs \\- an advanced revision control system\n"++
    ".SH SYNOPSIS\n"++
    ".B darcs\n"++
    "\\fICOMMAND \\fR...\n"++
@@ -60,7 +81,6 @@ man_header = do
    "see the html documentation, which should be available at\n"++
    datadir++"/doc/darcs/manual/index.html.  To easily get more specific help\n"++
    "on each command, you can call `darcs COMMAND --help'\n"++
-   "\n"++
-   ".SH COMMANDS\n"
+   "\n"
 
 
