@@ -1,24 +1,12 @@
-# LaTeX2HTML 2K.1beta (1.48)
+# LaTeX2HTML 2002-2-1 (1.70)
 # Associate labels original text with physical files.
 
 
-$key = q/merge_thm/;
+$key = q/merge_dfn/;
 $external_labels{$key} = "$URL/" . q|node8.html|; 
 $noresave{$key} = "$nosave";
 
-$key = q/configuring/;
-$external_labels{$key} = "$URL/" . q|node5.html|; 
-$noresave{$key} = "$nosave";
-
-$key = q/author_prefs/;
-$external_labels{$key} = "$URL/" . q|node5.html|; 
-$noresave{$key} = "$nosave";
-
-$key = q/annotate/;
-$external_labels{$key} = "$URL/" . q|node6.html|; 
-$noresave{$key} = "$nosave";
-
-$key = q/web_interface/;
+$key = q/selecting/;
 $external_labels{$key} = "$URL/" . q|node7.html|; 
 $noresave{$key} = "$nosave";
 
@@ -26,100 +14,72 @@ $key = q/defaults/;
 $external_labels{$key} = "$URL/" . q|node5.html|; 
 $noresave{$key} = "$nosave";
 
-$key = q/unrevert/;
+$key = q/dft-development-practices/;
 $external_labels{$key} = "$URL/" . q|node6.html|; 
 $noresave{$key} = "$nosave";
 
-$key = q/repository_format/;
-$external_labels{$key} = "$URL/" . q|node9.html|; 
+$key = q/merge_thm/;
+$external_labels{$key} = "$URL/" . q|node8.html|; 
+$noresave{$key} = "$nosave";
+
+$key = q/disk-usage/;
+$external_labels{$key} = "$URL/" . q|node4.html|; 
 $noresave{$key} = "$nosave";
 
 $key = q/gpl/;
 $external_labels{$key} = "$URL/" . q|node10.html|; 
 $noresave{$key} = "$nosave";
 
-$key = q/whatsnew/;
+$key = q/darcsssh/;
+$external_labels{$key} = "$URL/" . q|node5.html|; 
+$noresave{$key} = "$nosave";
+
+$key = q/darcs-development-practices/;
 $external_labels{$key} = "$URL/" . q|node6.html|; 
 $noresave{$key} = "$nosave";
 
-$key = q/resolution/;
-$external_labels{$key} = "$URL/" . q|node6.html|; 
-$noresave{$key} = "$nosave";
-
-$key = q/Patch/;
-$external_labels{$key} = "$URL/" . q|node8.html|; 
-$noresave{$key} = "$nosave";
-
-$key = q/merge_dfn/;
-$external_labels{$key} = "$URL/" . q|node8.html|; 
-$noresave{$key} = "$nosave";
-
-$key = q/token_replace/;
-$external_labels{$key} = "$URL/" . q|node8.html|; 
-$noresave{$key} = "$nosave";
-
-1;
-
-
-# LaTeX2HTML 2K.1beta (1.48)
-# labels from external_latex_labels array.
-
-
-$key = q/merge_thm/;
-$external_latex_labels{$key} = q|2|; 
-$noresave{$key} = "$nosave";
-
-$key = q/configuring/;
-$external_latex_labels{$key} = q|4|; 
-$noresave{$key} = "$nosave";
-
 $key = q/author_prefs/;
-$external_latex_labels{$key} = q|4.1.3|; 
-$noresave{$key} = "$nosave";
-
-$key = q/annotate/;
-$external_latex_labels{$key} = q|5.27|; 
-$noresave{$key} = "$nosave";
-
-$key = q/web_interface/;
-$external_latex_labels{$key} = q|6|; 
-$noresave{$key} = "$nosave";
-
-$key = q/defaults/;
-$external_latex_labels{$key} = q|4.1.1|; 
+$external_labels{$key} = "$URL/" . q|node5.html|; 
 $noresave{$key} = "$nosave";
 
 $key = q/unrevert/;
-$external_latex_labels{$key} = q|5.17|; 
-$noresave{$key} = "$nosave";
-
-$key = q/repository_format/;
-$external_latex_labels{$key} = q|B|; 
-$noresave{$key} = "$nosave";
-
-$key = q/gpl/;
-$external_latex_labels{$key} = q|C|; 
-$noresave{$key} = "$nosave";
-
-$key = q/whatsnew/;
-$external_latex_labels{$key} = q|5.5|; 
-$noresave{$key} = "$nosave";
-
-$key = q/resolution/;
-$external_latex_labels{$key} = q|5.1.9|; 
-$noresave{$key} = "$nosave";
-
-$key = q/Patch/;
-$external_latex_labels{$key} = q|A|; 
-$noresave{$key} = "$nosave";
-
-$key = q/merge_dfn/;
-$external_latex_labels{$key} = q|3|; 
+$external_labels{$key} = "$URL/" . q|node7.html|; 
 $noresave{$key} = "$nosave";
 
 $key = q/token_replace/;
-$external_latex_labels{$key} = q|A.5|; 
+$external_labels{$key} = "$URL/" . q|node8.html|; 
+$noresave{$key} = "$nosave";
+
+$key = q/repository_format/;
+$external_labels{$key} = "$URL/" . q|node9.html|; 
+$noresave{$key} = "$nosave";
+
+$key = q/annotate/;
+$external_labels{$key} = "$URL/" . q|node7.html|; 
+$noresave{$key} = "$nosave";
+
+$key = q/Patch/;
+$external_labels{$key} = "$URL/" . q|node8.html|; 
+$noresave{$key} = "$nosave";
+
+$key = q/resolution/;
+$external_labels{$key} = "$URL/" . q|node7.html|; 
+$noresave{$key} = "$nosave";
+
+$key = q/configuring/;
+$external_labels{$key} = "$URL/" . q|node5.html|; 
+$noresave{$key} = "$nosave";
+
+$key = q/whatsnew/;
+$external_labels{$key} = "$URL/" . q|node7.html|; 
 $noresave{$key} = "$nosave";
 
 1;
 
+
+# LaTeX2HTML 2002-2-1 (1.70)
+# labels from external_latex_labels array.
+
+
+1;
+
