@@ -1,5 +1,5 @@
 import System
-import Workaround ( matchRegex, mkRegex )
+import Text.Regex ( matchRegex, mkRegex )
 
 import DarcsCommands
 import DarcsArguments
@@ -9,14 +9,38 @@ import Autoconf ( darcs_version )
 main :: IO ()
 main = do
   args <- getArgs
-  if length args /= 1
+  if length args < 1
      then exitWith $ ExitFailure 1
      else return ()
   c <- preproc ["\\input{"++head args++"}"]
-  putStr $ unlines c
+  mapM_ putStrLn c
+
+am_html :: IO Bool
+am_html = do args <- getArgs
+             case args of
+               [_,"--html"] -> return True
+               _ -> return False
 
 preproc :: [String] -> IO [String]
+preproc ("\\usepackage{html}":ss) = -- only use html package with latex2html
+    do rest <- preproc ss
+       ah <- am_html
+       if ah then return $ "\\package{html}" : rest
+             else return $ "\\newcommand{\\htmladdnormallink}[2]{#1}" :
+                  "\\newcommand{\\htmladdnormallinkfoot}[2]{#1\\footnotetext{\\tt #2}}"
+                  : rest
 preproc ("\\begin{code}":ss) = ignore ss
+preproc ("\\begin{options}":ss) =
+    do rest <- preproc ss
+       ah <- am_html
+       if ah then return $ "\\begin{rawhtml}" : "<div class=\"cmd-opt-hdr\">" : rest
+             else return $ ("\\begin{Verbatim}[frame=lines,xleftmargin=1cm," ++
+                            "xrightmargin=1cm]") : rest
+preproc ("\\end{options}":ss) =
+    do rest <- preproc ss
+       ah <- am_html
+       if ah then return $ "</div>" : "\\end{rawhtml}" : rest
+             else return $ "\\end{Verbatim}" : rest
 preproc (s:ss) = do
   rest <- preproc ss
   case matchRegex (mkRegex "^\\\\input\\{(.+)\\}$") s of
@@ -51,7 +75,12 @@ get_c comm (c:cs) | command_name c == comm = c
 get_c comm [] = error $ "No such command:  "++comm
 
 get_com_options :: DarcsCommand -> String
-get_com_options c = options_latex $ command_darcsoptions c
+get_com_options c =
+    "\\verb!Usage: darcs " ++ cmd ++ " [OPTION]... " ++ args ++ "!\n\n" ++
+    "Options:\n\n" ++ opts
+    where cmd = command_name c
+          args = unwords $ command_extra_arg_help c
+          opts = options_latex $ command_darcsoptions c
 
 ignore :: [String] -> IO [String]
 ignore ("\\end{code}":ss) = preproc ss
