@@ -1,19 +1,20 @@
 #!/bin/sh
 set -ev
 
-DARCS=$PWD/../darcs
+test $DARCS || DARCS=$PWD/../darcs
 rm -rf temp1 temp2
 mkdir temp1 temp2
 cd temp1
-$DARCS inittree
+$DARCS init
 mkdir dir
 echo zig > dir/foo
 $DARCS add dir dir/foo
 #sleep 1
 $DARCS record -a -m add_foo -A x
 cd ../temp2
-$DARCS inittree
+$DARCS init
 $DARCS pull -a ../temp1
 cd ..
 cmp temp1/dir/foo temp2/dir/foo
 rm -rf temp1 temp2
+
