@@ -1,17 +1,17 @@
 #!/bin/sh
 set -ev
 
-DARCS=$PWD/../darcs
+test $DARCS || DARCS=$PWD/../darcs
 
 rm -rf tempA tempB
 mkdir tempA tempB
 cd tempA
-$DARCS inittree
+$DARCS init
 touch foo
 $DARCS add foo
 $DARCS record -a -m addA -A x
 cd ../tempB
-$DARCS inittree
+$DARCS init
 $DARCS pull -a ../tempA
 $DARCS remove foo
 # rm foo
@@ -23,3 +23,4 @@ cd ../tempB
 $DARCS pull -a ../tempA
 cd ..
 rm -rf tempA tempB
+
