@@ -1,11 +1,11 @@
 #!/bin/sh
 set -ev
 
-DARCS=$PWD/../darcs
-rm -rf foo1 foo2
-mkdir foo1
-cd foo1
-$DARCS inittree
+test $DARCS || DARCS=$PWD/../darcs
+rm -rf temp1 temp2
+mkdir temp1
+cd temp1
+$DARCS init
 echo record author me > _darcs/prefs/defaults
 echo ALL all >> _darcs/prefs/defaults
 echo ALL verbose >> _darcs/prefs/defaults
@@ -16,14 +16,15 @@ $DARCS record -m add_foo
 echo hello >> foo
 $DARCS record -m mod_foo
 cd ..
-$DARCS get -o foo2 foo1
-cd foo2
-cp ../foo1/_darcs/prefs/defaults _darcs/prefs
+$DARCS get --repo-name temp2 temp1
+cd temp2
+cp ../temp1/_darcs/prefs/defaults _darcs/prefs
 echo y | $DARCS unpull
 test -e foo -a ! -s foo
 echo hello >> foo
 $DARCS record -m mod_foo_again
-$DARCS pull ../foo1
+$DARCS pull ../temp1
 test -s foo
 cd ..
-rm -rf foo1 foo2
+rm -rf temp1 temp2
+
